﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

using Autodesk.Revit.DB;
using Autodesk.Revit.DB.Mechanical;
using Autodesk.Revit.UI;
using Autodesk.Revit.Attributes;
using Autodesk.Revit.UI.Selection;

namespace PressureLoss_v1
{
    [Transaction(TransactionMode.Manual)]
    public class PressureLoss : IExternalCommand
    {

/*************************************************** CESTA K SOUBORU .TXT ***************************************************/

        string path = @"C:\Users\micha\AppData\Roaming\Autodesk\Revit\Addins\2018\Koeficienty.txt";    // ZDE ZMĚNIT CESTU K SOUBORU

/*************************************************** CESTA K SOUBORU .TXT ***************************************************/

        public class VZTFilter : ISelectionFilter
        {
            //Ošetření, že nelze vybrat jiný elemnt než PŘÍMÉ POTRUBÍ a POTRUBNÍ TVAROVKA
            public bool AllowElement(Element elem)
            {
                if (elem.Category.Id.IntegerValue.Equals((int)BuiltInCategory.OST_DuctCurves) || elem.Category.Id.IntegerValue.Equals((int)BuiltInCategory.OST_DuctFitting))
                {
                    return true;
                }
                else return false;
            }

            public bool AllowReference(Reference reference, XYZ position)
            {
                return false;
            }
        }

        public string CalculatePressure (string RD)
        {
            double RDdbl = double.Parse(RD);
            double helpcoef = (9.69 * RDdbl);
            double helpcoef2 = helpcoef - 4.24;
            
            double helpcoef3 = Math.Pow(helpcoef2, -1);
            double Eta = RDdbl * helpcoef3;
            return Eta.ToString();
        }

        public string GetLine (string partType, string RD, string path)
        {

            int counter = 1;
            string coefOutput;
            string line;
            
            // Read the file and display it line by line.
            StreamReader file = new StreamReader(path);
            while ((line = file.ReadLine()) != null)
            {
                if (line.Contains(partType))
                {                                               
                    string linePositionInput = File.ReadLines(path).Skip(counter).Take(5).First();
                    int charPositionInput = linePositionInput.IndexOf(RD);                                         

                    if (linePositionInput.Contains(RD))
                    {
                        string linePositionOutput = File.ReadLines(path).Skip(counter + 1).Take(5).First();
                        coefOutput = linePositionOutput.Substring(charPositionInput, 4);
                        return coefOutput;
                    }
                    else
                    {
                        TaskDialog.Show("Chyba vstupního parametru", "Pro daný poměr R/D nebyla nalezena odpovídající hodnota koeficientu. R/D je mimo tabelární hodnoty");
                    }
                } 
                counter++;
                TaskDialog.Show("Chyba vstupního parametru", "Textový soubor neobsahuje příslešnou vzduchotechnickou tvarovkou (nenalezen PartType)");
                return null;
            }
            TaskDialog.Show("Chyba souboru", "Textový soubor Koeficienty.txt je prázdný");
            return null;
        }

        public Result Execute(ExternalCommandData commandData, ref string message, ElementSet elements)
        {
            UIApplication uiApp = commandData.Application;
            TaskDialog.Show("Aplikace PressureLoss Loss", "Oblastním výběrem vyberte prvky VZTm u nichž chcete zjistit tlakovou ztrát. Aplikace je takovým pilotním projektem, umí tedy počítat tlakové ztráty na kruhovém potrubí a kruhových kolenech.");

            try
            {
                // Nastavení vybírání objektů. Můžou se vybírat jenom VZT potrubí a pouze čtvercovým výběrem
                IList<Element> pickedRef = null;
                Document doc = uiApp.ActiveUIDocument.Document;
                Selection sel = uiApp.ActiveUIDocument.Selection;
                VZTFilter selFilter = new VZTFilter();
                pickedRef = sel.PickElementsByRectangle(selFilter, "Vyberte potrubní systém").Select(a => doc.GetElement(a.Id)).Cast<Element>().ToList();

                // Pro každý vybraný element se zjistí hodnota jeho parametru, která se uloží do Listu pressureDuct a rozdělí se na číselnou část (bez jednotky Pa) a změní se desetinná tečka za čárku, aby mohl systém sčítat tyto čísla
                // z LookUp bylo zjištěno, že parametr má hodnotu String (AsValueString), proto se hodnoty ukládají do stringového Listu
                List<string> pressureDuct = new List<string>();
                List<double> pressureDuctFitting = new List<double>();
                List<double> pressureDuctFittingEquation = new List<double>();
                List<Connector> lcon;

                Connector c0;
                Connector c1;

                double velocityPressure0;
                double velocityPressure1;

                foreach (Element elem in pickedRef)
                {
                    try
                    {
                        if (elem is FamilyInstance fi)
                        {
                            if (fi.MEPModel is MechanicalFitting)
                            {
                                // Získání hodnoty PartType z instance
                                MechanicalFitting mfit = fi.MEPModel as MechanicalFitting;
                                PartType pt = mfit.PartType;

                                // Získání parametru "CR" jakožto Central Radius. Takto pojmenovaný musí být v rodině Elbow, aby kalkulus fungoval
                                // Získání parametru vnitřního průměru
                                string radius = fi.LookupParameter("CR").AsValueString().Replace(".", ",").Split(' ')[0];
                                string diameter = elem.get_Parameter(BuiltInParameter.RBS_CALCULATED_SIZE).AsString().Split(' ')[0];
                                
                                // Získání VelocityPressure z Connector instance
                                lcon = (fi.MEPModel as MechanicalFitting).ConnectorManager.Connectors.Cast<Connector>().ToList();
                                c0 = lcon[0];
                                c1 = lcon[1];

                                velocityPressure0 = c0.VelocityPressure;
                                velocityPressure1 = c1.VelocityPressure;
                                  
                                // Výpočet R/D a získání hodnot z textového souboru
                                decimal RDdbl = decimal.Round(decimal.Parse(radius) / decimal.Parse(diameter), 2, MidpointRounding.AwayFromZero);
                                string RDstr = RDdbl.ToString();

                                try
                                {
                                        // HODNOTY Z TEXŤÁKU
                                    double num = double.Parse(GetLine(Enum.GetName(typeof(PartType), pt), RDstr, path));
                                    double pressureLosss = num * velocityPressure0;

                                    pressureDuctFitting.Add(pressureLosss);
                                }
                                catch
                                {
                                    TaskDialog.Show("Něco je špatně", "Textový soubor nebyl nalezen, ve Visual Studiu prosím vyplňte na řádku 21 správnou cestu k souboru a pomocí tlačítka 'Sestavit' a 'Sestavit řešení' vytvoříte nový *.dll soubor, který vožíte do C:\\Users\\XXXXX\\AppData\\Roaming\\Autodesk\\Revit\\Addins\\201Y");
                                }

                                // HODNOTY ZE VZTAHU
                                double num2 = double.Parse(CalculatePressure(RDstr));
                                double pressureLosss2 = num2 * velocityPressure0;

                                pressureDuctFittingEquation.Add(pressureLosss2);
                            }
                        }
                        else
                        {
                            if (elem is Duct d)
                            {
                                pressureDuct.Add(elem.get_Parameter(BuiltInParameter.RBS_PRESSURE_DROP).AsValueString().Replace(".", ",").Split(' ')[0]);
                            }
                        }
                    }
                    catch
                    {
                        TaskDialog.Show("Něco je špatně", "Nebyly vybrány elementy odpovídající vzduchotechnickýcm prvkům s parametry RBS_PRESSURE_DROP a PartType, nebo bylo vybráno čtvercové potrubí");
                    }
                }

                // Získaný parametr nemá vhodný tvar, je potřeba z něj oddělit jednotku a převést na double pro další výpočty + kontrola zdali není List prázdný
                List<double> pressureDuctDbl = new List<double>();
                if (pressureDuct.Count != 0)
                {
                    for (int i = 0; i < pressureDuct.Count; i++)
                    {
                        double split = double.Parse(pressureDuct[i]);
                        pressureDuctDbl.Add(split);
                    }
                }
                else
                {
                    pressureDuctDbl.Add(0);
                }

                // Sečtení hodnot v čísleném Listu a vypsání do textového řetězce         
                StringBuilder st = new StringBuilder();
                string strlossFitting;
                string strlossFitting2;
                string strLossDuct = Math.Round(pressureDuctDbl.Sum(), 3).ToString();

                // Ošetření proti prázdnému Listu
                if (pressureDuctFitting.Count != 0)
                {
                    st.AppendLine("Hodnoty z textového souboru");
                    st.AppendLine("Tlaková ztráta přímých úseků: " + strLossDuct + " Pa");
                    strlossFitting = Math.Round(pressureDuctFitting.Sum(), 3).ToString();
                    st.AppendLine("Tlaková ztráta tvarovek: " + strlossFitting + " Pa");
                    double dbllossCelk = double.Parse(strLossDuct) + double.Parse(strlossFitting);
                    st.AppendLine("Celková tlaková ztráta: " + dbllossCelk.ToString() + " Pa");

                    if (pressureDuctFittingEquation.Count != 0)
                    {
                        st.AppendLine();
                        st.AppendLine("Hodnoty z rovnice");
                        st.AppendLine("Tlaková ztráta přímých úseků: " + strLossDuct + " Pa");
                        strlossFitting2 = Math.Round(pressureDuctFittingEquation.Sum(), 3).ToString();
                        st.AppendLine("Tlaková ztráta tvarovek: " + strlossFitting2 + " Pa");
                        double dbllossCelk2 = double.Parse(strLossDuct) + double.Parse(strlossFitting2);
                        st.AppendLine("Celková tlaková ztráta: " + dbllossCelk2.ToString() + " Pa");
                    }
                    else
                    {
                        pressureDuctFittingEquation.Add(0);
                        st.AppendLine();
                        st.AppendLine("Hodnoty z rovnice");
                        st.AppendLine("Tlaková ztráta přímých úseků: " + strLossDuct + " Pa");
                        strlossFitting2 = Math.Round(pressureDuctFittingEquation.Sum(), 3).ToString();
                        st.AppendLine("Tlaková ztráta tvarovek: " + strlossFitting2 + " Pa");
                        double dbllossCelk2 = double.Parse(strLossDuct) + double.Parse(strlossFitting2);
                        st.AppendLine("Celková tlaková ztráta: " + dbllossCelk2.ToString() + " Pa");
                    }
                }
                else
                {
                    pressureDuctFitting.Add(0);
                    st.AppendLine("Hodnoty z textového souboru");
                    st.AppendLine("Tlaková ztráta přímých úseků: " + strLossDuct + " Pa");
                    strlossFitting = Math.Round(pressureDuctFitting.Sum(), 3).ToString();
                    st.AppendLine("Tlaková ztráta tvarovek: " + strlossFitting + " Pa");
                    double dbllossCelk = double.Parse(strLossDuct) + double.Parse(strlossFitting);
                    st.AppendLine("Celková tlaková ztráta: " + dbllossCelk.ToString() + " Pa");

                    if (pressureDuctFittingEquation.Count != 0)
                    {
                        st.AppendLine();
                        st.AppendLine("Hodnoty z rovnice");
                        st.AppendLine("Tlaková ztráta přímých úseků: " + strLossDuct + " Pa");
                        strlossFitting2 = Math.Round(pressureDuctFittingEquation.Sum(), 3).ToString();
                        double dbllossCelk2 = double.Parse(strLossDuct) + double.Parse(strlossFitting2);
                        st.AppendLine("Celková tlaková ztráta: " + dbllossCelk2.ToString() + " Pa");
                    }
                    else
                    {
                        pressureDuctFittingEquation.Add(0);
                        st.AppendLine();
                        st.AppendLine("Hodnoty z rovnice");
                        st.AppendLine("Tlaková ztráta přímých úseků: " + strLossDuct + " Pa");
                        strlossFitting2 = Math.Round(pressureDuctFittingEquation.Sum(), 3).ToString();
                        double dbllossCelk2 = double.Parse(strLossDuct) + double.Parse(strlossFitting2);
                        st.AppendLine("Celková tlaková ztráta: " + dbllossCelk2.ToString() + " Pa");
                    }
                }

                // Zobrazení okna s textovým řetězcem výsledku
                // Podmínka, aby byl vybrán alespon jeden prvek, jinak systém jede dál (ukončí příkaz)
                if (pickedRef.Count != 0)
                {
                    TaskDialog.Show("Pressure loss", st.ToString());
                }
                else
                {
                    return Result.Cancelled;
                }

            return Result.Succeeded;
            }

            // Jednání v případě, že příkaz bude ukončen
            catch (Autodesk.Revit.Exceptions.OperationCanceledException)
            {
                TaskDialog.Show("Dvojka", "Dvojka");
                return Result.Cancelled;
            }

            // Jednání v případě, že nastane chyba
            catch (Exception ex)
            {
                TaskDialog.Show("Trojka", "Trojka");
                message = ex.Message;
                return Result.Failed;
            }
        }
    }
}
